#version 330 core
layout(location=0) in vec3 deltaVertex;
layout(location=1) in vec3 vertex;
layout(location=2) in vec3 animvertex;
uniform mat4 M;
uniform mat4 V;
uniform mat4 P;
uniform float iGlobalTime;
uniform float fisheye;
uniform float particlesize;

uniform int animated;
uniform float frame;

out vec2 UV;

vec2 hash( vec2 p ) {  						// rand in [-1,1]
	p = vec2( dot(p,vec2(127.1,311.7)),
			  dot(p,vec2(269.5,183.3)) );
	return -1. + 2.*fract(sin(p+20.)*53758.5453123);
}

float noise( in vec2 p ) {
    vec2 i = floor(p), f = fract(p);
	vec2 u = f*f*(3.-2.*f);
    return mix( mix( dot( hash( i + vec2(0.,0.) ), f - vec2(0.,0.) ), 
                     dot( hash( i + vec2(1.,0.) ), f - vec2(1.,0.) ), u.x),
                mix( dot( hash( i + vec2(0.,1.) ), f - vec2(0.,1.) ), 
                     dot( hash( i + vec2(1.,1.) ), f - vec2(1.,1.) ), u.x), u.y);
}

void main() {
	//gl_Position = P * V * M * vec4(vertex.xy + 0.1*(deltaVertex.xy * (1.0 + 0.4*noise(24579.0*vertex.xy + 0.5 * iGlobalTime))), 0.0, 1.0);
	vec3 radial = vertex.xyz;
	if (animated == 1) {
		radial = frame * vertex.xyz + (1.0 - frame) * animvertex;
	}
	radial = radial * (0.9 + 0.2 * noise(vec2(0.5 * iGlobalTime + vertex.y + vertex.z, vertex.x)));
	vec3 cu = vec3(V[0][0], V[1][0], V[2][0]);
	vec3 cr = vec3(V[0][1], V[1][1], V[2][1]);
	vec4 viewTfd = V * M * vec4(radial + 3.0 * particlesize*(deltaVertex.x * cu + deltaVertex.y * cr), 1.0);
	vec4 projd = P * viewTfd;
	gl_Position = (1.0 - fisheye) * projd + fisheye * vec4(viewTfd.xy / length(viewTfd.xyz), projd.zw);
	UV = deltaVertex.xy + vec2(0.5);
}
